# Programm zur Loesung der Aufgabe 10.2
# mit Dirichlet- und Neumann-RB, iterative Loesung
# input: n,m Stuetzpunkte in x- und y-Richtung, 
#        omega Relaxationsparameter, epsilon Genauigkeit 
# output: Loesung u(x,y) auf dem Gitter x,y, 
#         udiff Abstand zweier Loesungen, it Iterationszahl
# Aufruf: [u,x,y,udiff,it] = aufg92i(n,m,omega,epsilon,itmax)
function [u,x,y,udiff,it] = aufg92i(n,m,omega,epsilon,itmax);
# Materialdaten, Quellen
la = 3.0; ff=0;
# Gebeitslaenge, Gebietshoehe
ll = 2; hh = 1;
# Dimensionierung des Gitters
dx = ll/n; dy = hh/m;
h = dx; k = dy;
# Gitterinitialisierung
x(1) = dx/2; y(1) = dy/2;
for i=1:n
  x(i) = (i-1)*dx+x(1);
endfor
for j=1:m
  y(j) = (j-1)*dy+y(1);
endfor
# Startfeld
for i=1:n
  for j=1:m
    u(i,j) = 1.0;
  endfor
endfor 
# Konstanten
h2=h^2; k2=k^2; dx2=2*dx; dy2=2*dy;
udiff = 1; unorm = 1; it = 0;
# Randwerte
uw=1; q1=0; q2=0; q3=0;
# Aufbau der Matrix und der rechten Seite
for i=1:n
    for j=1:m
      ind = i + (j-1)*n;
      vm = (x(i)-0.5*dx)^2; vp = (x(i)+0.5*dx)^2;
      wm = (1-y(j)+0.5*dy)^2; wp = (1-y(j)-0.5*dy)^2;
      avm = abs(vm); avp = abs(vp);
      awm = abs(wm); awp = abs(wp);
      vpp = 0.5*(vp+avp); vpm = 0.5*(vp-avp);
      vmp = 0.5*(vm+avm); vmm = 0.5*(vm-avm);
      wpp = 0.5*(wp+awp); wpm = 0.5*(wp-awp);
      wmp = 0.5*(wm+awm); wmm = 0.5*(wm-awm);
      if (1 < i && i < n && 1 < j && j < m )
        a(ind,ind) = 2*la/h2+2*la/k2 + (vpp-vmm)/dx + (wpp-wmm)/dy;
        a(ind,ind+1) = -(la/h2 - vpm/dx);
        a(ind,ind-1) = -(la/h2 + vmp/dx);
        a(ind,ind+n) = -(la/k2 - wpm/dy);
        a(ind,ind-n) = -(la/k2 + wmp/dy);
        ff1(ind) = ff;
      endif
      if (i == 1 && 1 < j && j < m )
        a(ind,ind) = 3*la/h2+2*la/k2 + (vpp-vmm+vmp)/dx + (wpp-wmm)/dy;
        a(ind,ind+1) = -(la/h2 - vpm/dx);
        a(ind,ind+n) = -(la/k2 - wpm/dy);
        a(ind,ind-n) = -(la/k2 + wmp/dy);
        ff1(ind) = ff-2*vpm*uw/h+la*2*uw/h2;
      endif
      if (1 < i && i < n && j == 1 )
        a(ind,ind) = 2*la/h2+la/k2 + (vpp-vmm)/dx + (wpp - wmm - wmp)/dy;
        a(ind,ind+1) = -(la/h2 - vpm/dx);
        a(ind,ind-1) = -(la/h2 + vmp/dx);
        a(ind,ind+n) = -(la/k2 - wpm/dy);
        ff1(ind) = ff+wmp*q1+la*q1/k;
      endif
      if (i == n && 1 < j && j < m )
        a(ind,ind) = la/h2+2*la/k2 + (vpp - vmm + vpm)/dx + (wpp-wmm)/dy;
        a(ind,ind-1) = -(la/h2 + vmp/dx);
        a(ind,ind+n) = -(la/k2 - wpm/dy);
        a(ind,ind-n) = -(la/k2 + wmp/dy);
        ff1(ind) = ff-vpm*q2+la*q2/h;
      endif
      if (1 < i && i < n && j == m )
        a(ind,ind) = 2*la/h2+la/k2 + (vpp-vmm)/dx + (wpp - wmm + wpm)/dy;
        a(ind,ind+1) = -(la/h2 - vpm/dx);
        a(ind,ind-1) = -(la/h2 + vmp/dx);
        a(ind,ind-n) = -(la/k2 + wmp/dy);
        ff1(ind) = ff-wpm*q3+la*q3/k;
      endif
      if (i == 1 && j == 1 )
        a(ind,ind) = 3*la/h2+la/k2 + (vpp - vmm + vmp)/dx +(wpp - wmm- wmp)/dy;
        a(ind,ind+1) = -(la/h2 - vpm/dx);
        a(ind,ind+n) = -(la/k2 - wpm/dy);
        ff1(ind) = ff+2*vmp*uw/h+wmp*q1+la*2*uw/h2+la*q1/k;
      endif
      if (i == 1 && j == m )
        a(ind,ind) = 3*la/h2+la/k2 + (vpp - vmm + vmp)/dx + (wpp - wmm + wpm)/dy;
        a(ind,ind+1) = -(la/h2 - vpm/dx);
        a(ind,ind-n) = -(la/k2 + wmp/dy);
        ff1(ind) = ff + 2*vmp*uw/h - wpm*q3 + la*2*uw/h2 +la*q3/k;
      endif
      if (i == n && j == 1 )
        a(ind,ind) = la/h2+la/k2 + (vpp - vmm + vpm)/dx + (wpp - wmm - wmp)/dy;
        a(ind,ind-1) = -(la/h2 + vmp/dx);
        a(ind,ind+n) = -(la/k2 - wpm/dy);
        ff1(ind) = ff-vpm*q2+wmp*q1+la*q2/h+la*q1/k;
      endif
      if (i == n && j == m )
        a(ind,ind) = la/h2+la/k2 + (vpp - vmm + vpm)/dx + (wpp - wmm + wpm)/dy;
        a(ind,ind-1) = -(la/h2 + vmp/dx);
        a(ind,ind-n) = -(la/k2 + wmp/dy);
        ff1(ind) = ff-vmp*q2-wmp*q3+la*q2/h+la*q3/k;
      endif
    endfor
endfor
# sor-Iteration
while (sqrt(udiff/unorm) > epsilon && it < itmax)
  ualt = u;
  udiff
  it
  for i=1:n
    for j=1:m
      ind = i + (j-1)*n;
      if (1 < i && i < n && 1 < j && j < m )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind-1)*u(i-1,j) ...
               -a(ind,ind+n)*u(i,j+1)-a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      endif
      if (i == 1 && 1 < j && j < m )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind+n)*u(i,j+1) ...
               -a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      endif
      if (1 < i && i < n && j == 1 )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind-1)*u(i-1,j) ...
               -a(ind,ind+n)*u(i,j+1))/a(ind,ind);
      endif
      if (i == n && 1 < j && j < m )
        u(i,j)=(ff1(ind)-a(ind,ind-1)*u(i-1,j)-a(ind,ind+n)*u(i,j+1) ...
               -a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      endif
      if (1 < i && i < n && j == m )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind-1)*u(i-1,j) ...
               -a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      endif
      if (i == 1 && j == 1 )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind+n)*u(i,j+1))/a(ind,ind);
      endif
      if (i == 1 && j == m )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      endif
      if (i == n && j == 1 )
        u(i,j)=(ff1(ind)-a(ind,ind-1)*u(i-1,j)-a(ind,ind+n)*u(i,j+1))/a(ind,ind);
      endif
      if (i == n && j == m )
        u(i,j)=(ff1(ind)-a(ind,ind-1)*u(i-1,j)-a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      endif
    endfor
  endfor
  udiff = 0.0; unorm = 0.0;
  for i=1:n
    for j=1:m
      u(i,j) = omega*u(i,j)+(1-omega)*ualt(i,j);
      udiff = udiff + (u(i,j)-ualt(i,j))^2;
      unorm = unorm + ualt(i,j)^2;
    endfor
  endfor
  if (unorm == 0.0)
    unorm = 1;
  endif
  it = it + 1;
endwhile
endfunction
