# Programm zum Simplexverfahren der linearen Optimierung/Programmierung
# Berechnet die Loesung X eines linearen Programms in der Standardform (*)
#   C'X -> min (Zielfunktion)
#   A*X=B
#   X>=0
#   B>=0
#
# Eingabe: A = m x n - Matrix
#          B = m - Vektor
#          C = n - Vektor
# Ausgabe: X = n - Spaltenvektor
#
# Aufruf: X=simplex_o(A,B,C)
function X=simplex_o(A,B,C);
# 19.6.95 Bjoern Harzer
eps = 1.0e-10;
[m,n] = size(A);
B = B(:); 
C = C(:);
b = length(B);
c = length(C);
if (b~=m | c~=n)
     disp('error in format');
     return 
endif
# Eine Basisloesung des Problems (**)
#   y(1)+...+y(m) -> min
#   A*X+Y=B
#   X,Y>=0
# ist [X;Y] = [0;B]
II = n+1:n+m;               # Indexmenge
X = B;                      # Basisloesung
U = eye(m);                 # Inverse zu a(:,II)
c = [zeros(n,1);ones(m,1)]; # Schlupfvariablen einfuehren
a = [A,eye(m)];
# Phase1: optimale Loesung von (**) bestimmen
K = 1:n+m; K(II) = 0*II; K = find(K~=0);
D = c(K)-a(:,K)'*U'*c(II);
while any(D<-eps)
  disp('Phase1');
  [min1,k]=min(D);
  k=K(k);
  Z=U*a(:,k);
  ZI=find(Z>eps);
  P=X(ZI)./Z(ZI);
  [min1,l] = min(P);
  j=II(ZI(l));
  Uneu=U(ZI(l),:)./Z(ZI(l));
  U=U-Z*Uneu;
  U(ZI(l),:)=Uneu;
  Xneu=X(ZI(l))./Z(ZI(l));
  X=X-Z*Xneu;
  X(ZI(l))=Xneu;
  II(ZI(l))=k;
  K = 1:n+m; K(II) = 0*II; K = find(K~=0);
  D = c(K)-a(:,K)'*U'*c(II);
endwhile
# Phase2: optimale Loesung von (**) gleich Basisloesung von (*),
#         nun optimale Loesung von (*) bestimmen
K = 1:n; K(II) = 0*II; K = find(K~=0);
for jj=1:length(II)
  if (II(jj) > length(C))
    disp('Problem nicht loesbar, Nebenbedingungsmenge ist leer!');
    X = []; 
    return
  endif
endfor
D = C(K)-A(:,K)'*U'*C(II);
while any(D<-eps)
  disp('Phase2');
  [min1,k]=min(D);
  k=K(k);
  Z=U*A(:,k);
  ZI=find(Z>eps);
  if isempty(ZI)
     X=inf*C(:);
     disp('Zielfunktion unbeschraenkt!');
     return
  endif	
  P=X(ZI)./Z(ZI);
  [min1,l] = min(P);
  j=II(ZI(l));
  Uneu=U(ZI(l),:)./Z(ZI(l));
  U=U-Z*Uneu;
  U(ZI(l),:)=Uneu;
  Xneu=X(ZI(l))./Z(ZI(l));
  X=X-Z*Xneu;
  X(ZI(l))=Xneu;
  II(ZI(l))=k;
  K = 1:n; K(II) = 0*II; K = find(K~=0);
  D = C(K)-A(:,K)'*U'*C(II);
endwhile
Y = X;
X = zeros(n,1);
X(II) = Y;
endfunction
# Ende des Programms
